/***************/
/* Pietrek.cpp */
/***************/

// Files:

// DependencyList.cpp
// MemoryMappedFile.cpp
// PeSection.cpp
// Exefile.cpp
// PeExe.cpp
// PeExe2.cpp

#define PIETREK_ALL

#include <windows.h>
#include <malloc.h>
#include <stdlib.h>

#include "Pietrek.h"

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: DEPENDENCYLIST
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#include <malloc.h>
#pragma hdrstop
#include "peexe.h"
#include "dependencylist.h"
#endif

MODULE_DEPENDENCY_LIST::MODULE_DEPENDENCY_LIST( PSTR pszFileName )
{
	m_cModules = 0;
	m_pList = 0;

	// Make a copy of the path that we can modify to get just the path portion
	PSTR pszJustPath = _strdup( pszFileName );
	if ( !pszJustPath )
		return;
	
	BOOL fHasPath = FALSE;
	PSTR pszEnd = strrchr( pszJustPath, '\\' );
	if ( pszEnd )
	{
		*pszEnd = 0;	/// Strip off the filename
		fHasPath = TRUE;
	}

	//
	// If a path was part of the input filename, save the current directory,
	// then switch to the new directory.
	//	
	char szOriginalPath[MAX_PATH];
	if ( fHasPath )
	{
		// This doesn't take into account "App_Paths"!
		GetCurrentDirectory(MAX_PATH, szOriginalPath);	// Save original dir
		SetCurrentDirectory( pszJustPath );				// Switch to app's dir
	}

	//
	// recursively build the module list
	//
	AddModule( pszFileName );
	
	if ( fHasPath )		// Set things back to the way they were
		SetCurrentDirectory( szOriginalPath );
	
	free( pszJustPath );	// Free the copy of the path that we allocated
}


MODULE_DEPENDENCY_LIST::~MODULE_DEPENDENCY_LIST( )
{
	PMODULE_FILE_INFO pTemp;

	pTemp = m_pList;

	// Delete each of the MODULE_FILE_INFO structures in the linked list
	while ( pTemp )
	{
		pTemp = m_pList->m_pNext;
		delete m_pList;
		m_pList = pTemp;
		m_cModules--;
	}

	m_pList = 0;
}


PMODULE_FILE_INFO MODULE_DEPENDENCY_LIST::GetNextModule( PMODULE_FILE_INFO p )
{
	// Returns the next module in the linked list of MODULE_FILE_INFO's
	return p ? p->m_pNext : m_pList;
}


// Given the name of a file, find the MODULE_FILE_INFO structure that
// represents it.  The fFullName parameter specifies whether the full path
// names or just the base file names will be compared.
PMODULE_FILE_INFO MODULE_DEPENDENCY_LIST::LookupModule( PSTR pszFileName,
														BOOL fFullName )
{
	PMODULE_FILE_INFO p = m_pList;	// Start at the list head

	while ( p )	// While there's still entries in the list...
	{
		PSTR pszCompName = fFullName ? p->m_szFullName : p->m_szBaseName;

		if ( 0 == lstrcmpi( pszFileName, pszCompName ) )
			return p;

		p = p->m_pNext;
	}

	return 0;
}

// Adds a modules to the MODULE_FILE_INFO list.  If the module imports other
// modules, this routine recurses to add them, and check their imports.
void MODULE_DEPENDENCY_LIST::AddModule( PSTR pszFullName )
{
	PE_EXE peFile( pszFullName );		// Get easy access to the executable
	
	if ( FALSE == peFile.IsValid() )	// A valid PE file???
		return;

	// Find the last '\\' to obtain a pointer to just the base filename part
	PSTR pszBaseName = strrchr( pszFullName, '\\' );
	if ( pszBaseName )	// We found a path, so advance to the base filename
		pszBaseName++;
	else
		pszBaseName = pszFullName;	// No path.  Use the same name for both
	
	PMODULE_FILE_INFO pNew = new MODULE_FILE_INFO;
	if ( !pNew )
		return;
	
	// Initialize the new MODULE_FILE_INFO, and stick it at the head
	// of the list.
	lstrcpyn( pNew->m_szFullName, pszFullName, sizeof(pNew->m_szFullName) );
	lstrcpyn( pNew->m_szBaseName, pszBaseName, sizeof(pNew->m_szBaseName) );
	pNew->m_pNext = m_pList;
	m_pList = pNew;

	m_cModules++;
	
	//
	// Now see if this module imports any other modules.  If so, we need
	// to recurse and add them as well.
	//
	if (0 == peFile.GetDataDirectoryEntrySize( IMAGE_DIRECTORY_ENTRY_IMPORT ))
		return;

	// Make a pointer to the imports table
	PIMAGE_IMPORT_DESCRIPTOR pImportDir;
	pImportDir = (PIMAGE_IMPORT_DESCRIPTOR)
		peFile.GetDataDirectoryEntryPointer(IMAGE_DIRECTORY_ENTRY_IMPORT);
	if ( !pImportDir )
		return;

	// While there are still non-null IMAGE_IMPORT_DESCRIPTORs...
	while ( pImportDir->Name )
	{
		// Get a pointer to the imported module's base name
		PSTR pszBaseName;
		pszBaseName = (PSTR)peFile.GetReadablePointerFromRVA(pImportDir->Name);
		if ( !pszBaseName )
			break;

		// Check to see if it's already in our list.  Don't add again if so.
		if ( 0 == LookupModule( pszBaseName, FALSE ) )
		{
			// Search path supposedly has the same searching algorithm as
			// the the Win32 loader...
			char szPath[MAX_PATH];
			PSTR pszDontCare;
			if ( SearchPath(0, pszBaseName, 0, MAX_PATH, szPath, &pszDontCare))
				AddModule( szPath );
		}
		
		pImportDir++;	// Advance to next imported module
	}
}

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: MEMORYMPAPPEDFILE.CPP
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#pragma hdrstop
#include "memorymappedfile.h"
#endif

MEMORY_MAPPED_FILE::MEMORY_MAPPED_FILE( PSTR pszFileName )
{
	//
	// Given a filename, the constructor opens a file handle, creates a file
	// mapping, and maps the entire file into memory.
	//
	m_hFile = INVALID_HANDLE_VALUE;
	m_hFileMapping = 0;
	m_pMemoryMappedFileBase = 0;
	m_cbFile = 0;
	m_errCode = errMMF_NoError;

	// First get a file handle		
	m_hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
						OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)0);
					
	if ( m_hFile == INVALID_HANDLE_VALUE )
	{
		m_errCode = errMMF_FileOpen;
		return;
	}

	m_cbFile = ::GetFileSize( m_hFile, 0 );

	// Now, create a file mapping	
	m_hFileMapping = CreateFileMapping(m_hFile,NULL, PAGE_READONLY, 0, 0,NULL);
	if ( m_hFileMapping == 0 )
	{
		// Oops.  Something went wrong.  Clean up.
		CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
		m_errCode = errMMF_FileMapping;
		return;
	}

	m_pMemoryMappedFileBase = (PCHAR)MapViewOfFile( m_hFileMapping,
													FILE_MAP_READ, 0, 0, 0);

	if ( m_pMemoryMappedFileBase == 0 )
	{
		// Oops.  Something went wrong.  Clean up.
		CloseHandle(m_hFileMapping);
		m_hFileMapping = 0;
		CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
		m_errCode = errMMF_MapView;
		return;
	}
}

MEMORY_MAPPED_FILE::~MEMORY_MAPPED_FILE(void)
{
	// Clean up everything that was created by the constructor
	if ( m_pMemoryMappedFileBase )
		UnmapViewOfFile( m_pMemoryMappedFileBase );

	if ( m_hFileMapping )
		CloseHandle( m_hFileMapping );

	if ( m_hFile != INVALID_HANDLE_VALUE )
		CloseHandle( m_hFile );	
}

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PESECTION.CPP
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#include <stdlib.h>
#pragma hdrstop
#include "pesection.h"
#endif

PE_SECTION::PE_SECTION( PIMAGE_SECTION_HEADER pSectHdr,
						PVOID pMappedAddr,
						unsigned index )
{
	m_pSectHdr = pSectHdr;
	m_pMappedAddr = pMappedAddr;
	m_index = index;
}

void PE_SECTION::GetName( PSTR pszName )
{
	//  input buffer must be at least IMAGE_SIZEOF_SHORT_NAME +1 bytes

	// Copy the max name size, even if the string is less than the max
	// length.  Next, null terminate it.
	memcpy( pszName, m_pSectHdr->Name, IMAGE_SIZEOF_SHORT_NAME );
	pszName[ IMAGE_SIZEOF_SHORT_NAME ] = 0;
}

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: EXEFILE.CPP
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#pragma hdrstop
#include "exefile.h"
#endif

EXE_FILE::EXE_FILE( PSTR pszFileName ) : MEMORY_MAPPED_FILE( pszFileName )
{
	m_secondaryHeaderOffset = -1;	// A bogus value to catch bugs
	m_exeType = exeType_Invalid;

	if ( FALSE == MEMORY_MAPPED_FILE::IsValid() )
		return;
	
	if ( GetFileSize() < sizeof(IMAGE_DOS_HEADER) )
		return;
	
	PIMAGE_DOS_HEADER pDosHdr = (PIMAGE_DOS_HEADER)GetBase();
	if ( IMAGE_DOS_SIGNATURE != pDosHdr->e_magic )
		return;
	
	if ( pDosHdr->e_lfarlc < 0x40 )	// Theoretically, this field must be >=
	{								// 0x40 for it to be a non-DOS executable
		m_exeType = exeType_DOS;
		return;
	}

	// Sanity check.  Make sure the "new header" offset isn't past the end
	// of the file
	if ( pDosHdr->e_lfanew > (LONG)GetFileSize() )
		return;

	// Make a pointer to the secondary header	
	m_secondaryHeaderOffset = pDosHdr->e_lfanew;
	PWORD pSecondHdr = MakePtr( PWORD, GetBase(), m_secondaryHeaderOffset );

	// Decide what type of EXE, based on the start of the secondary header
	switch ( *pSecondHdr )
	{
		case IMAGE_OS2_SIGNATURE: m_exeType = exeType_NE; break;
		case IMAGE_VXD_SIGNATURE: m_exeType = exeType_VXD; break;
		case 0x4558: m_exeType = exeType_LX; break;		// OS/2 2.X
	}
	
	if ( *(PDWORD)pSecondHdr == IMAGE_NT_SIGNATURE )
		m_exeType = exeType_PE;
}

PSTR EXE_FILE::GetFileTypeDescription( void )
{
	// Returns a static string that describes what type this file is
	switch ( m_exeType )
	{
		case exeType_DOS:	return "DOS";
		case exeType_NE:	return "NE";
		case exeType_VXD:	return "VXD";
		case exeType_LX:	return "LX";
		case exeType_PE:	return "PE";
		default:			return "Invalid";
	}
}

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PEEXE.CPP
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#include <stdlib.h>
#pragma hdrstop
#include "peexe.h"
#endif

PE_EXE::PE_EXE( PSTR pszFileName ) : EXE_FILE( pszFileName )
{
	m_pNtHdr = 0;
	
	if ( GetExeType() != exeType_PE )	// If not a PE file, bail out
		return;

	m_pNtHdr = MakePtr(PIMAGE_NT_HEADERS,GetBase(),GetSecondaryHeaderOffset());
}

DWORD PE_EXE::GetDataDirectoryEntryRVA( DWORD id )
{
	// Given a IMAGE_DIRECTORY_ENTRY_XXX value (see WINNT.H), retrive the
	// RVA stored in the corresponding slot

	if ( id >= IMAGE_NUMBEROF_DIRECTORY_ENTRIES )
		return (DWORD)-1;

	return m_pNtHdr->OptionalHeader.DataDirectory[id].VirtualAddress;
}

PVOID PE_EXE::GetDataDirectoryEntryPointer( DWORD id )
{
	// Given a IMAGE_DIRECTORY_ENTRY_XXX value (see WINNT.H), return a pointer
	// to memory that corresponds to the RVA in the specified slot.

	if ( id >= IMAGE_NUMBEROF_DIRECTORY_ENTRIES )
		return (PVOID)-1;
	
	DWORD va = m_pNtHdr->OptionalHeader.DataDirectory[id].VirtualAddress;

	if ( !va )		// Return 0 if the RVA is 0
		return 0;

	return GetReadablePointerFromRVA( va );
}

DWORD PE_EXE::GetDataDirectoryEntrySize( DWORD id )
{
	// Given a IMAGE_DIRECTORY_ENTRY_XXX value (see WINNT.H), retrive the
	// size value stored in the corresponding slot

	if ( id >= IMAGE_NUMBEROF_DIRECTORY_ENTRIES )
		return (DWORD)-1;

	return m_pNtHdr->OptionalHeader.DataDirectory[id].Size;
}

PVOID PE_EXE::GetReadablePointerFromRVA( DWORD rva )
{
	// Given an RVA, translate it into a pointer within our linear memory
	// mapping for the executable.

	DWORD fileOffset = RVAToFileOffset( rva );
	if ( (DWORD)-1 == fileOffset )
		return 0;
	
	return MakePtr( PVOID, GetBase(), fileOffset );
}

DWORD PE_EXE::RVAToFileOffset( DWORD rva )
{
	// Given an RVA, figure out which section encompasses it.  Next, using
	// the PointerToRawData field for the found section, return an actual
	// file offset that corresponds to the RVA

	PIMAGE_SECTION_HEADER pSectHdr = IMAGE_FIRST_SECTION( m_pNtHdr );
	
	for ( unsigned i = 0; i < GetNumberOfSections(); i++, pSectHdr++ )
	{
		DWORD cbMaxOnDisk
			= min( pSectHdr->Misc.VirtualSize, pSectHdr->SizeOfRawData );

		DWORD startSectRVA = pSectHdr->VirtualAddress;
		DWORD endSectRVA = startSectRVA + cbMaxOnDisk;
		
		if ( (rva >= startSectRVA) && (rva < endSectRVA) )
			return pSectHdr->PointerToRawData + (rva - startSectRVA);
	}
	
	return (DWORD)-1;	// RVA not found in the section table... Ooops!
}

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PEEXE2.CPP
//==========================================

#ifndef PIETREK_ALL
#include <windows.h>
#include <stdlib.h>
#pragma hdrstop
#include "peexe2.h"
#endif

PE_EXE2::PE_EXE2( PSTR pszFileName ) : PE_EXE( pszFileName )
{
	m_ppPeSect = 0;		// Start out with no section table
}

PE_EXE2::~PE_EXE2( )
{
	unsigned i;

	// Clean up the PE_SECTIONs (if we created them)
	if ( m_ppPeSect )
	{
		for ( i = 1; i <= GetNumberOfSections(); i++ )
			delete m_ppPeSect[i];
		
		delete []m_ppPeSect;
	}
}

PE_SECTION * PE_EXE2::GetSectionByName( PSTR pszName )
{
	if ( !m_ppPeSect )	// Make the section table (if not already built)
		MakeSections();	

	char szThisSectName[ IMAGE_SIZEOF_SHORT_NAME ];
	
	for ( unsigned i = 1; i <= GetNumberOfSections(); i++ )
	{
		m_ppPeSect[i]->GetName( szThisSectName );
		if ( 0 == lstrcmp( pszName, szThisSectName) )
			return m_ppPeSect[i];
	}
	
	return 0;
}

PE_SECTION * PE_EXE2::GetSectionByIndex( unsigned index )	// 1 based
{
	if ( index > GetNumberOfSections() )	// Is it a valid section number
		return 0;

	if ( 0 == index )	// Zero is not a valid section number!!!
		return 0;

	if ( !m_ppPeSect )	// Make the section table (if not already built)
		MakeSections();
	
	return m_ppPeSect ? m_ppPeSect[ index ] : 0;
}

PE_SECTION * PE_EXE2::GetSectionByRVA( DWORD rva )
{
	if ( !m_ppPeSect )	// Make the section table (if not already built)
		MakeSections();

	// Iterate through each section looking for one that encompasses the
	// passed in rva.  If found, return a pointer to the PE_SECTION object
	for ( unsigned i = 1; i <= GetNumberOfSections(); i++ )
	{
		PE_SECTION * pSection = m_ppPeSect[i];

		if ( rva >= pSection->GetVirtualAddress() )
			if ( rva < (  pSection->GetVirtualAddress()
						+ pSection->GetMaxSizeInMem() ))
			{
				return pSection;
			}
	}

	return 0;	// Oops.  Not found.
}

unsigned PE_EXE2::GetDebugInfoTypeCount( void )
{
	DWORD cbDebugInfo;

	// Go to the data directory to see how big the debug directory is	
	cbDebugInfo = GetDataDirectoryEntrySize( IMAGE_DIRECTORY_ENTRY_DEBUG );
	if ( !cbDebugInfo )
		return 0;

	// Microsoft puts in the size of the debug directory.  Borland emits
	// the actual number of debug types (i.e., 1 for Borland)	
	if ( cbDebugInfo >= sizeof(IMAGE_DEBUG_DIRECTORY) )
		return cbDebugInfo / sizeof( IMAGE_DEBUG_DIRECTORY );
	else
		return cbDebugInfo;		// Borland has this screwed up!
}

PIMAGE_DEBUG_DIRECTORY PE_EXE2::GetDebugDirectory( void )
{
	// returns a pointer to the array of IMAGE_DEBUG_DIRECTORYs
	if ( GetDebugInfoTypeCount() )	// Is there debug info in the first place?
	{
		return (PIMAGE_DEBUG_DIRECTORY)
				GetDataDirectoryEntryPointer( IMAGE_DIRECTORY_ENTRY_DEBUG );
	}
	else
		return 0;
}

BOOL PE_EXE2::MakeSections( void )
{
	unsigned i;

	// A helper routine, which is called the first time that somebody
	// asks for section related information.

	// Allocate space for our array of pointers to PE_SECTIONs
	// Make space for one extra pointer at the beginning to be "section 0",
	// although this "section" will never be used.
	m_ppPeSect = new PE_SECTION *[ GetNumberOfSections()+1 ];

	PIMAGE_SECTION_HEADER pRawSects
					= IMAGE_FIRST_SECTION( GetIMAGE_NT_HEADERS() );
	
	for ( i = 1; i <= GetNumberOfSections(); i++ )
		m_ppPeSect[i] = new PE_SECTION( pRawSects++, GetBase(), i );
			
	return FALSE;
}
