/*************/
/* Pietrek.h */
/*************/

// Files:

// DependencyList.h
// MemoryMappedFile.h
// PeSection.h
// Exefile.h
// PeExe.h
// PeExe2.h

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: DEPENDENCYLIST.CPP
//==========================================
#ifndef __DEPLIST_H__
#define __DEPLIST_H__

//
// This structure represents one executable file in a module dependency list.
// Both the base filename and the complete path are stored.
//
struct MODULE_FILE_INFO
{
    MODULE_FILE_INFO * m_pNext;
    char    m_szBaseName[MAX_PATH];
    char    m_szFullName[MAX_PATH];
};
typedef MODULE_FILE_INFO * PMODULE_FILE_INFO;

//
// The MODULE_DEPENDENCY_LIST class creates a linked list of MODULE_FILE_INFO
// structures.  In theory, this list will represent every executable file
// loaded by the Win32 loader when the executable is loaded.  The class creates
// the list by starting with the file passed to the constructor, and recursing
// through all the import tables.
//
class MODULE_DEPENDENCY_LIST
{
    public:
        
    MODULE_DEPENDENCY_LIST( PSTR pszFileName );
    ~MODULE_DEPENDENCY_LIST( );
    
    PMODULE_FILE_INFO GetNextModule( PMODULE_FILE_INFO p );
        
    PMODULE_FILE_INFO LookupModule( PSTR pszFileName, BOOL fFullName );

    unsigned GetNumberOfModules( void ){ return m_cModules; }

    protected:
        
    unsigned            m_cModules;     // Number of modules in list
    PMODULE_FILE_INFO   m_pList;        // Pointer to head of linked list
    
    void AddModule( PSTR pszFullName ); // Recursively adds modules to the list
};

#endif

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: MEMORYMAPPEDFILE.H
//==========================================
#ifndef __MEMMAPFL_H__
#define __MEMMAPFL_H__

enum errMMF {	errMMF_NoError, errMMF_FileOpen,
				errMMF_FileMapping, errMMF_MapView };

class MEMORY_MAPPED_FILE
{
	public:
	
	MEMORY_MAPPED_FILE( PSTR pszFileName );
	~MEMORY_MAPPED_FILE(void);
	
	PVOID	GetBase( void ){ return m_pMemoryMappedFileBase; }
	DWORD	GetFileSize( void ){ return m_cbFile; }
	BOOL	IsValid( void ) { return errMMF_NoError == m_errCode; } 

	private:

	HANDLE		m_hFile;
	HANDLE		m_hFileMapping;	// Handle of memory mapped file
	PVOID		m_pMemoryMappedFileBase;
	DWORD		m_cbFile;
	unsigned	m_errCode;	
};

typedef MEMORY_MAPPED_FILE *PMEMORY_MAPPED_FILE;

#endif

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PESECTION.H
//==========================================
#ifndef __PESECT_H__
#define __PESECT_H__

#ifndef MakePtr
#define MakePtr( cast, ptr, addValue ) (cast)( (DWORD)(ptr) + (DWORD)(addValue))
#endif

class PE_SECTION
{
	public:
		
	PE_SECTION( PIMAGE_SECTION_HEADER pSectHdr,
				PVOID pMappedAddr,
				unsigned index );
	~PE_SECTION( ) { }

	//
	// Functions for getting at the raw data of an IMAGE_SECTION_HEADER
	//
    void    GetName( PSTR pszName );	// IMAGE_SIZEOF_SHORT_NAME +1 bytes

    DWORD   GetPhysicalAddress( void )
				{ return m_pSectHdr->Misc.PhysicalAddress; }
    DWORD   GetVirtualSize( void )
				{ return m_pSectHdr->Misc.VirtualSize; }
    DWORD   GetVirtualAddress( void )
				{ return m_pSectHdr->VirtualAddress; }
    DWORD   GetSizeOfRawData( void )
				{ return m_pSectHdr->SizeOfRawData; }
    DWORD   GetPointerToRawData( void )		// Really a file offset!
				{ return m_pSectHdr->PointerToRawData; }
    DWORD   GetPointerToRelocations( void )
				{ return m_pSectHdr->PointerToRelocations; }
    DWORD   GetPointerToLinenumbers( void )
				{ return m_pSectHdr->PointerToLinenumbers; }
    WORD    GetNumberOfRelocations( void )
				{ return m_pSectHdr->NumberOfRelocations; }
    WORD    GetNumberOfLinenumbers( void )
				{ return m_pSectHdr->NumberOfLinenumbers; }
    DWORD   GetCharacteristics( void )
				{ return m_pSectHdr->Characteristics; }

	//
	// More intelligent functions that interpret the IMAGE_SECTION_HEADER data
	//

	PVOID	GetPointer( void )	// Returns the *REAL* pointer to raw data
				{ return MakePtr(PVOID, m_pMappedAddr, GetPointerToRawData()); }

	DWORD	GetAccessibleSize( void )
				{ return min(	m_pSectHdr->Misc.VirtualSize,
								m_pSectHdr->SizeOfRawData ); }

	DWORD	GetMaxSizeInMem( void )
				{ return max(	m_pSectHdr->Misc.VirtualSize,
								m_pSectHdr->SizeOfRawData ); }
							
	BOOL	IsCode( void )
				{ return m_pSectHdr->Characteristics & IMAGE_SCN_MEM_EXECUTE; }
	BOOL	IsData( void )
				{ return !IsCode(); }
	BOOL	IsWriteable( void )
				{ return m_pSectHdr->Characteristics & IMAGE_SCN_MEM_WRITE; }
	
	DWORD	GetIndex( void ) { return m_index; }

	private:
	
	PIMAGE_SECTION_HEADER m_pSectHdr;
	PVOID m_pMappedAddr;
	unsigned m_index;
};

#endif

//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: EXEFILE.H
//==========================================
#ifndef __EXEFILE_H__
#define __EXEFILE_H__

//#ifndef __MEMMAPFL_H__
//#include "memorymappedfile.h"
//#endif

// MakePtr is a macro that allows you to easily add to values (including
// pointers) together without dealing with C's pointer arithmetic.  It
// essentially treats the last two parameters as DWORDs.  The first
// parameter is used to typecast the result to the appropriate pointer type.
#define MakePtr( cast, ptr, addValue ) (cast)( (DWORD)(ptr) + (DWORD)(addValue))

enum EXE_TYPE { exeType_Invalid, exeType_DOS, exeType_NE, exeType_VXD,
				exeType_LX, exeType_PE };

class EXE_FILE : public MEMORY_MAPPED_FILE
{
	public:
		
	EXE_FILE( PSTR pszFileName );
	~EXE_FILE( ){ ; }
	
	BOOL	IsValid( void ){ return m_exeType != exeType_Invalid; }
	
	DWORD	GetSecondaryHeaderOffset( void ){ return m_secondaryHeaderOffset; }

	EXE_TYPE	GetExeType( void ){ return m_exeType; }

	PSTR	GetFileTypeDescription( void );
	
	private:

	LONG		m_secondaryHeaderOffset;
	EXE_TYPE	m_exeType;
};

#endif
//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PEEXE.H
//==========================================
#ifndef __PEEXE_H__
#define __PEEXE_H__

//#ifndef __EXEFILE_H__
//#include "exefile.h"
//#endif

class PE_EXE : public EXE_FILE
{
	public:

	PE_EXE( PSTR pszFileName );
	~PE_EXE( ){ }

	int IsValid() { return m_pNtHdr ? TRUE : FALSE; }

	// For those who want at the data directly
	PIMAGE_NT_HEADERS GetIMAGE_NT_HEADERS( void ) { return m_pNtHdr; }

	// IMAGE_FILE_HEADER fields
	WORD	GetMachine( void )
				{ return m_pNtHdr->FileHeader.Machine; }
	WORD	GetNumberOfSections( void )
				{ return m_pNtHdr->FileHeader.NumberOfSections; }
	DWORD	GetTimeDateStamp(void)
				{ return m_pNtHdr->FileHeader.TimeDateStamp; }
	DWORD	GetCharacteristics( void )
				{ return m_pNtHdr->FileHeader.Characteristics; }

	// IMAGE_OPTIONAL_HEADER fields
	DWORD	GetSizeOfCode( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfCode; }
    DWORD   GetSizeOfInitializedData( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfInitializedData; }
    DWORD   GetSizeOfUninitializedData( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfUninitializedData; }
    DWORD   GetAddressOfEntryPoint( void )
			{ return m_pNtHdr->OptionalHeader.AddressOfEntryPoint; }
    DWORD   GetBaseOfCode( void )
			{ return m_pNtHdr->OptionalHeader.BaseOfCode; }
    DWORD   GetBaseOfData( void )
			{ return m_pNtHdr->OptionalHeader.BaseOfData; }
    DWORD   GetImageBase( void )
			{ return m_pNtHdr->OptionalHeader.ImageBase; }
    DWORD   GetSectionAlignment( void )
			{ return m_pNtHdr->OptionalHeader.SectionAlignment; }
    DWORD   GetFileAlignment( void )
			{ return m_pNtHdr->OptionalHeader.FileAlignment; }
    WORD    GetMajorOperatingSystemVersion( void )
			{ return m_pNtHdr->OptionalHeader.MajorOperatingSystemVersion; }
    WORD    GetMinorOperatingSystemVersion( void )
			{ return m_pNtHdr->OptionalHeader.MinorOperatingSystemVersion; }
    WORD    GetMajorImageVersion( void )
			{ return m_pNtHdr->OptionalHeader.MajorImageVersion; }
    WORD    GetMinorImageVersion( void )
			{ return m_pNtHdr->OptionalHeader.MinorImageVersion; }
    WORD    GetMajorSubsystemVersion( void )
			{ return m_pNtHdr->OptionalHeader.MajorSubsystemVersion; }
    WORD    GetMinorSubsystemVersion( void )
			{ return m_pNtHdr->OptionalHeader.MinorSubsystemVersion; }
//    DWORD   GetWin32VersionValue( void )
//			{ return m_pNtHdr->OptionalHeader.Win32VersionValue; }
    DWORD   GetSizeOfImage( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfImage; }
    DWORD   GetSizeOfHeaders( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfHeaders; }
    WORD    GetSubsystem( void )
			{ return m_pNtHdr->OptionalHeader.Subsystem; }
    DWORD   GetSizeOfStackReserve( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfStackReserve; }
    DWORD   GetSizeOfStackCommit( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfStackCommit; }
    DWORD   GetSizeOfHeapReserve( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfHeapReserve; }
    DWORD   GetSizeOfHeapCommit( void )
			{ return m_pNtHdr->OptionalHeader.SizeOfHeapCommit; }

	DWORD	GetDataDirectoryEntryRVA( DWORD id );
	PVOID	GetDataDirectoryEntryPointer( DWORD id );
	DWORD	GetDataDirectoryEntrySize( DWORD id );

	PVOID	GetReadablePointerFromRVA( DWORD rva );
	
	protected:

	DWORD	RVAToFileOffset( DWORD rva );

	PIMAGE_NT_HEADERS m_pNtHdr;
};

#endif
//==========================================
// LIPO32/LIPOSUCTION32 - Matt Pietrek 1996
// Microsoft Systems Journal, October 1996
// FILE: PEEXE2.H
//==========================================
#ifndef __PEEXE2_H__
#define __PEEXE2_H__

//#ifndef __PEEXE_H__
//#include "peexe.h"
//#endif

//#ifndef __PESECT_H__
//#include "pesection.h"
//#endif

class PE_EXE2 : public PE_EXE
{
	public:
		
	PE_EXE2( PSTR pszFileName );
	~PE_EXE2( );

	PE_SECTION * GetSectionByName( PSTR pszName );
	PE_SECTION * GetSectionByIndex( unsigned index );
	PE_SECTION * GetSectionByRVA( DWORD RVA );

	unsigned GetDebugInfoTypeCount( void );
	PIMAGE_DEBUG_DIRECTORY GetDebugDirectory( void );

	private:
		
	PE_SECTION ** m_ppPeSect;
	
	BOOL MakeSections( void );
};

#endif
