/***************/
/* exe2kos.cpp */
/***************/

#include <windows.h>
#include <imagehlp.h>
#include <stdio.h>

#include "Pietrek.h"

struct kos_Header
{
    char kosSign[8];
    DWORD res1;
    DWORD start;
    DWORD end;
    DWORD memory;
    DWORD stack;
    DWORD res2;
    DWORD res3;

    kos_Header()
    {
        memcpy(kosSign, "MENUET01", 8);
        res1 = 1;
        res2 = 0;
        res3 = 0;
    }
};

#define SIZE_OF_HEADER sizeof(kos_Header)

int InFileProcessing(char *strInFile);
int OutFileProcessing(char *strInFile, char *strOutFile);
void OutputError(PSTR pszErrString);

int main(int argc, char *argv[])
{
    if (argc != 3)
    {
        printf("Syntax: exe2kos <inFilename> <outFile>\n");
        return 1;
    }

    if (InFileProcessing(argv[1]))
    {
        return 1;
    }

    if (OutFileProcessing(argv[1], argv[2]))
    {
        return 1;
    }

    return 0;
}

int InFileProcessing(char *strInFile)
{
    PE_SECTION *ptrSection;
    char szBuffer[MAX_PATH * 2];

    PE_EXE2 peFile(strInFile);

    if (!peFile.IsValid()) 
    {
        wsprintf(szBuffer, "%s is not a PE file", strInFile);
        OutputError(szBuffer);
        return 1;
    }

    if (IMAGE_FILE_MACHINE_I386 != peFile.GetMachine())
    {
        wsprintf(szBuffer, "%s is not an Intel i386 executable", strInFile);
        OutputError(szBuffer);
        return 1;
    }

    if (peFile.GetDataDirectoryEntrySize(IMAGE_DIRECTORY_ENTRY_BASERELOC) == 0)
    {
        wsprintf(szBuffer, "Input file does not have relocations");
        OutputError(szBuffer);
        return 1;
    }

    ptrSection = peFile.GetSectionByName(".text");

    if (!ptrSection)
    {
        wsprintf(szBuffer, "Input file does not have .text section");
        OutputError(szBuffer);
        return 1;
    }

    if (peFile.GetNumberOfSections() != 2)
    {
        wsprintf(szBuffer, "Input file must contain EXACTLY 2 sections");
        OutputError(szBuffer);
        return 1;
    }

    if (peFile.GetSectionAlignment() != 0x10000)
    {
        wsprintf(szBuffer, "Input file /ALIGN must be 0x10000");
        OutputError(szBuffer);
        return 1;
    }

    // Lame attempt at seeing that user reserved 36 bytes for OS header.

    if (strncmp((char *)ptrSection->GetPointer(), "KolibriOS Header", 16) != 0)
    {
        wsprintf(szBuffer, "KolibriOS header not found or in wrong place");
        OutputError(szBuffer);
        return 1;
    } 

    return 0;
}

int OutFileProcessing(char *strInFile, char *strOutFile)
{
    PE_SECTION *ptrSection;
    unsigned int dwKosEntry, dwFileSize;
    ULONG OldImageSize, OldImageBase, NewImageSize, NewImageBase = 0xFFFFFFFFFFFF0000;
    char szBuffer[MAX_PATH * 2];
    FILE *outFile;
    kos_Header hdr;

    if (!ReBaseImage(strInFile, NULL, TRUE, FALSE, FALSE, 0,
                     &OldImageSize, &OldImageBase, &NewImageSize, &NewImageBase, NULL))
    {
        wsprintf(szBuffer, "ReBaseImage failed.");
        OutputError(szBuffer);
        return 1;
    }

    PE_EXE2 peFile(strInFile);

    ptrSection = peFile.GetSectionByName(".text");
    dwFileSize = ptrSection->GetAccessibleSize();
    dwKosEntry = peFile.GetAddressOfEntryPoint() - 0x10000;

    outFile = fopen(strOutFile, "w+b");
    if (outFile == NULL)
    {
        wsprintf(szBuffer, "Open of %s for output failed\n", strOutFile);
        OutputError(szBuffer);
        return 1;
    }

    hdr.start = dwKosEntry;
    hdr.end = dwFileSize;
    hdr.stack = dwFileSize + 4096;
    hdr.memory = hdr.stack;

    fwrite(&hdr, SIZE_OF_HEADER, 1, outFile);
    fwrite((char *)ptrSection->GetPointer() + SIZE_OF_HEADER, dwFileSize - SIZE_OF_HEADER, 1, outFile);

    fclose(outFile);

    return 0;
}

void OutputError(PSTR pszErrString)
{
    printf("%s\n", pszErrString);
}
